@@ -4,6 +4,7 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE IncoherentInstances #-}
+{-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE TypeSynonymInstances #-}
 
 -- Copyright (C) 2009-2012 John Millikin <john@john-millikin.com>
@@ -23,32 +24,35 @@
 
 module DBus.Internal.Types where
 
-import Control.DeepSeq
-import Control.Exception (Exception, handle, throwIO)
-import Control.Monad (liftM, when, (>=>))
-import Data.ByteString (ByteString)
-import Data.Char (ord)
-import Data.Int
-import Data.List (intercalate)
-import Data.Map (Map)
-import Data.Text (Text)
-import Data.Typeable (Typeable)
-import Data.Vector (Vector)
-import Data.Word
-import GHC.Generics
-import System.IO.Unsafe (unsafePerformIO)
-import System.Posix.Types (Fd)
-import Text.ParserCombinators.Parsec ((<|>), oneOf)
+import           Control.DeepSeq
+import           Control.Exception (Exception, handle, throwIO)
+import           Control.Monad (liftM, when, (>=>))
 import qualified Data.ByteString
+import           Data.ByteString (ByteString)
 import qualified Data.ByteString.Char8 as Char8
 import qualified Data.ByteString.Lazy
 import qualified Data.ByteString.Unsafe
+import           Data.Char (ord)
+import           Data.Coerce
+import           Data.Int
+import           Data.List (intercalate)
+import           Data.List.Split (splitOn)
 import qualified Data.Map
+import           Data.Map (Map)
 import qualified Data.String
 import qualified Data.Text
+import           Data.Text (Text)
 import qualified Data.Text.Lazy
+import           Data.Typeable (Typeable)
 import qualified Data.Vector
+import           Data.Vector (Vector)
+import           Data.Word
 import qualified Foreign
+import           GHC.Generics
+import qualified Language.Haskell.TH.Lift as THL
+import           System.IO.Unsafe (unsafePerformIO)
+import           System.Posix.Types (Fd)
+import           Text.ParserCombinators.Parsec ((<|>), oneOf)
 import qualified Text.ParserCombinators.Parsec as Parsec
 
 data Type
@@ -320,6 +324,10 @@ parseSigFull bytes = unsafePerformIO io where
                             return (ii' + 1, TypeDictionary t1 t2)
                         else throwIO SigParseError
 
+extractFromVariant :: IsValue a => Variant -> Maybe a
+extractFromVariant (Variant (ValueVariant v)) = extractFromVariant v
+extractFromVariant v = fromVariant v
+
 class IsVariant a where
     toVariant :: a -> Variant
     fromVariant :: Variant -> Maybe a
@@ -602,6 +610,17 @@ instance (Ord k, IsAtom k, IsValue v) => IsVariant (Map k v) where
     toVariant = Variant . toValue
     fromVariant (Variant val) = fromValue val
 
+instance IsValue () where
+  typeOf _ = TypeStructure []
+  toValue _ = ValueStructure []
+  fromValue (ValueStructure []) = return ()
+  fromValue _ = Nothing
+
+instance IsVariant () where
+  toVariant () = Variant (ValueStructure [])
+  fromVariant (Variant (ValueStructure [])) = Just ()
+  fromVariant _ = Nothing
+
 instance (IsValue a1, IsValue a2) => IsValue (a1, a2) where
     typeOf ~(a1, a2) = TypeStructure [typeOf a1, typeOf a2]
     toValue (a1, a2) = ValueStructure [toValue a1, toValue a2]
@@ -635,6 +654,12 @@ varToVal a = case toVariant a of
 newtype ObjectPath = ObjectPath String
     deriving (Eq, Ord, Show, NFData)
 
+pathElements :: ObjectPath -> [String]
+pathElements = filter (not . null) . splitOn "/" . coerce
+
+fromElements :: [String] -> ObjectPath
+fromElements elems = objectPath_ $ '/':intercalate "/" elems
+
 formatObjectPath :: ObjectPath -> String
 formatObjectPath (ObjectPath s) = s
 
@@ -1344,3 +1369,5 @@ maybeParseString :: Parsec.Parser a -> String -> Maybe a
 maybeParseString parser s = case Parsec.parse parser "" s of
     Left _ -> Nothing
     Right a -> Just a
+
+THL.deriveLiftMany [''BusName, ''ObjectPath, ''InterfaceName, ''MemberName]
