@@ -2,18 +2,17 @@
 
 -- Copyright (C) 2012 John Millikin <john@john-millikin.com>
 --
--- This program is free software: you can redistribute it and/or modify
--- it under the terms of the GNU General Public License as published by
--- the Free Software Foundation, either version 3 of the License, or
--- any later version.
+-- Licensed under the Apache License, Version 2.0 (the "License");
+-- you may not use this file except in compliance with the License.
+-- You may obtain a copy of the License at
 --
--- This program is distributed in the hope that it will be useful,
--- but WITHOUT ANY WARRANTY; without even the implied warranty of
--- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--- GNU General Public License for more details.
+--     http://www.apache.org/licenses/LICENSE-2.0
 --
--- You should have received a copy of the GNU General Public License
--- along with this program.  If not, see <http://www.gnu.org/licenses/>.
+-- Unless required by applicable law or agreed to in writing, software
+-- distributed under the License is distributed on an "AS IS" BASIS,
+-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+-- See the License for the specific language governing permissions and
+-- limitations under the License.
 
 module DBusTests.Transport (test_Transport) where
 
@@ -85,7 +84,7 @@ suite_OpenUnix = testGroup "unix"
 
 test_OpenUnix_Path :: TestTree
 test_OpenUnix_Path = testCase "path" $ runResourceT $ do
-    (addr, networkSocket) <- listenRandomUnixPath
+    addr <- listenRandomUnixPath
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
@@ -96,7 +95,7 @@ test_OpenUnix_Path = testCase "path" $ runResourceT $ do
 
 test_OpenUnix_Abstract :: TestTree
 test_OpenUnix_Abstract = testCase "abstract" $ runResourceT $ do
-    (addr, networkSocket, _) <- listenRandomUnixAbstract
+    (addr, _) <- listenRandomUnixAbstract
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
@@ -140,7 +139,7 @@ test_OpenUnix_NotListening :: TestTree
 test_OpenUnix_NotListening = testCase "not-listening" $ runResourceT $ do
     fdcountBefore <- countFileDescriptors
 
-    (addr, networkSocket, key) <- listenRandomUnixAbstract
+    (addr, key) <- listenRandomUnixAbstract
     release key
 
     liftIO $ assertThrows
@@ -166,7 +165,7 @@ suite_OpenTcp = testGroup "tcp"
 
 test_OpenTcp_IPv4 :: TestTree
 test_OpenTcp_IPv4 = testCase "ipv4" $ runResourceT $ do
-    (addr, networkSocket, _) <- listenRandomIPv4
+    (addr, _, _) <- listenRandomIPv4
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
@@ -177,7 +176,7 @@ test_OpenTcp_IPv4 = testCase "ipv4" $ runResourceT $ do
 
 test_OpenTcp_IPv6 :: TestTree
 test_OpenTcp_IPv6 = testCase "ipv6" $ unlessM noIPv6 $ runResourceT $ do
-    (addr, networkSocket) <- listenRandomIPv6
+    addr <- listenRandomIPv6
     fdcountBefore <- countFileDescriptors
 
     t <- liftIO (transportOpen socketTransportOptions addr)
@@ -260,7 +259,7 @@ test_OpenTcp_NotListening :: TestTree
 test_OpenTcp_NotListening = testCase "too-many" $ runResourceT $ do
     fdcountBefore <- countFileDescriptors
 
-    (addr, networkSocket, key) <- listenRandomIPv4
+    (addr, _, key) <- listenRandomIPv4
     release key
     liftIO $ assertThrows
         (\err -> and
@@ -282,7 +281,7 @@ test_TransportSendReceive = testCase "send-receive" $ runResourceT $ do
         fix $ \loop -> do
             bytes <- recv s 50
             if Data.ByteString.null bytes
-                then NS.sClose s
+                then NS.close s
                 else do
                     sendAll s bytes
                     loop
@@ -315,7 +314,7 @@ test_HandleLostConnection = testCase "handle-lost-connection" $ runResourceT $ d
     _ <- liftIO $ forkIO $ do
         (s, _) <- NS.accept networkSocket
         sendAll s "123"
-        NS.sClose s
+        NS.close s
 
     (_, t) <- allocate
         (transportOpen socketTransportOptions addr)
