@@ -1,17 +1,16 @@
 -- Copyright (C) 2010-2012 John Millikin <john@john-millikin.com>
 --
--- This program is free software: you can redistribute it and/or modify
--- it under the terms of the GNU General Public License as published by
--- the Free Software Foundation, either version 3 of the License, or
--- any later version.
+-- Licensed under the Apache License, Version 2.0 (the "License");
+-- you may not use this file except in compliance with the License.
+-- You may obtain a copy of the License at
 --
--- This program is distributed in the hope that it will be useful,
--- but WITHOUT ANY WARRANTY; without even the implied warranty of
--- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--- GNU General Public License for more details.
+--     http://www.apache.org/licenses/LICENSE-2.0
 --
--- You should have received a copy of the GNU General Public License
--- along with this program.  If not, see <http://www.gnu.org/licenses/>.
+-- Unless required by applicable law or agreed to in writing, software
+-- distributed under the License is distributed on an "AS IS" BASIS,
+-- WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+-- See the License for the specific language governing permissions and
+-- limitations under the License.
 
 module DBusTests.Util
     ( assertVariant
@@ -91,7 +90,7 @@ getTempPath = do
     uuid <- randomUUID
     return (tmp </> formatUUID uuid)
 
-listenRandomUnixPath :: MonadResource m => m (Address, N.Socket)
+listenRandomUnixPath :: MonadResource m => m Address
 listenRandomUnixPath = do
     path <- liftIO getTempPath
 
@@ -99,16 +98,16 @@ listenRandomUnixPath = do
     (_, sock) <- allocate
         (NS.socket NS.AF_UNIX NS.Stream NS.defaultProtocol)
         N.sClose
-    liftIO (NS.bindSocket sock sockAddr)
+    liftIO (NS.bind sock sockAddr)
     liftIO (NS.listen sock 1)
     _ <- register (removeFile path)
 
     let Just addr = address "unix" (Map.fromList
             [ ("path", path)
             ])
-    return (addr, sock)
+    return addr
 
-listenRandomUnixAbstract :: MonadResource m => m (Address, N.Socket, ReleaseKey)
+listenRandomUnixAbstract :: MonadResource m => m (Address, ReleaseKey)
 listenRandomUnixAbstract = do
     uuid <- liftIO randomUUID
     let sockAddr = NS.SockAddrUnix ('\x00' : formatUUID uuid)
@@ -117,13 +116,13 @@ listenRandomUnixAbstract = do
         (NS.socket NS.AF_UNIX NS.Stream NS.defaultProtocol)
         N.sClose
 
-    liftIO $ NS.bindSocket sock sockAddr
+    liftIO $ NS.bind sock sockAddr
     liftIO $ NS.listen sock 1
 
     let Just addr = address "unix" (Map.fromList
             [ ("abstract", formatUUID uuid)
             ])
-    return (addr, sock, key)
+    return (addr, key)
 
 listenRandomIPv4 :: MonadResource m => m (Address, N.Socket, ReleaseKey)
 listenRandomIPv4 = do
@@ -133,7 +132,7 @@ listenRandomIPv4 = do
     (key, sock) <- allocate
         (NS.socket NS.AF_INET NS.Stream NS.defaultProtocol)
         N.sClose
-    liftIO $ NS.bindSocket sock sockAddr
+    liftIO $ NS.bind sock sockAddr
     liftIO $ NS.listen sock 1
 
     sockPort <- liftIO $ NS.socketPort sock
@@ -144,7 +143,7 @@ listenRandomIPv4 = do
             ])
     return (addr, sock, key)
 
-listenRandomIPv6 :: MonadResource m => m (Address, N.Socket)
+listenRandomIPv6 :: MonadResource m => m Address
 listenRandomIPv6 = do
     addrs <- liftIO $ NS.getAddrInfo Nothing (Just "::1") Nothing
     let sockAddr = case addrs of
@@ -154,7 +153,7 @@ listenRandomIPv6 = do
     (_, sock) <- allocate
         (NS.socket NS.AF_INET6 NS.Stream NS.defaultProtocol)
         N.sClose
-    liftIO $ NS.bindSocket sock sockAddr
+    liftIO $ NS.bind sock sockAddr
     liftIO $ NS.listen sock 1
 
     sockPort <- liftIO $ NS.socketPort sock
@@ -163,7 +162,7 @@ listenRandomIPv6 = do
             , ("host", "::1")
             , ("port", show (toInteger sockPort))
             ])
-    return (addr, sock)
+    return addr
 
 noIPv6 :: IO Bool
 noIPv6 = do
